@@ -6,7 +6,7 @@ from functools import lru_cache
 from io import BytesIO
 from pathlib import Path
 from tokenize import tokenize, TokenInfo
-from typing import Optional, TypeVar, cast, Callable, List, Tuple, Dict, Type, Sequence, Union
+from typing import Optional, TypeVar, cast, Callable, List, Tuple, Dict, Type, Sequence, Union, Iterator
 
 from more_itertools import peekable
 
@@ -388,20 +388,25 @@ class ParserVisitor(ast.NodeVisitor):
     def visit_withitem(self, node):
         prefix = self.__whitespace()
         expr = self.__convert(node.context_expr)
-        value = self.__pad_left(self.__source_before('as'), expr)
-        name = self.__convert(node.optional_vars)
-        return j.Try.Resource(
-            random_id(),
-            prefix,
-            Markers.EMPTY,
-            j.Assignment(
+        if node.optional_vars:
+            value = self.__pad_left(self.__source_before('as'), expr)
+            name = self.__convert(node.optional_vars)
+            var = j.Assignment(
                 random_id(),
                 Space.EMPTY,
                 Markers.EMPTY,
                 name,
                 value,
                 self.__map_type(node.context_expr)
-            ),
+            )
+        else:
+            var = expr
+
+        return j.Try.Resource(
+            random_id(),
+            prefix,
+            Markers.EMPTY,
+            var,
             False
         )
 
@@ -1039,10 +1044,6 @@ class ParserVisitor(ast.NodeVisitor):
         if isinstance(node.func, ast.Name):
             select = None
             name = cast(j.Identifier, self.__convert(node.func))
-        elif isinstance(node.func, ast.Call):
-            select = self.__pad_right(cast(Expression, self.__convert(node.func)), self.__whitespace())
-            # printer handles empty name by not printing `.` before it
-            name = self.__convert_name('')
         elif isinstance(node.func, ast.Attribute):
             select = self.__pad_right(self.__convert(node.func.value), self.__source_before('.'))
             name = j.Identifier(
@@ -1054,8 +1055,12 @@ class ParserVisitor(ast.NodeVisitor):
                 self.__map_type(node.func.value),
                 None
             )
+        elif isinstance(node.func, (ast.Call, ast.Subscript)):
+            select = self.__pad_right(cast(Expression, self.__convert(node.func)), self.__whitespace())
+            # printer handles empty name by not printing `.` before it
+            name = self.__convert_name('')
         else:
-            raise NotImplementedError("Calls to functions other than methods are not yet supported")
+            raise NotImplementedError(f"Calls to functions other than methods are not yet supported: {type(node.func)}")
 
         all_args = node.args + node.keywords
         args = JContainer(
@@ -1153,13 +1158,28 @@ class ParserVisitor(ast.NodeVisitor):
 
 
     def visit_Constant(self, node):
-        # noinspection PyTypeChecker
+        prefix = self.__whitespace()
+
+        start = self._cursor
+        tokens = peekable(tokenize(BytesIO(self._source[self._cursor:].encode('utf-8')).readline))
+        tok = next(tokens)  # skip ENCODING token
+        tok = self.__next_lexer_token(tokens)
+
+        while tok.type == token.STRING:
+            while (tok := next(tokens)).type in (token.NEWLINE, token.INDENT, token.DEDENT):
+                pass
+            if tok.type == token.STRING:
+                self._cursor = self._source.index(tok.string, self._cursor)
+                self._cursor += len(tok.string)
+            else:
+                break
+
         return j.Literal(
             random_id(),
-            self.__whitespace(),
+            prefix,
             Markers.EMPTY,
             None if node.value is Ellipsis else node.value,
-            self.__next_lexer_token(),
+            self._source[start:self._cursor],
             None,
             self.__map_type(node),
         )
@@ -1434,7 +1454,7 @@ class ParserVisitor(ast.NodeVisitor):
                 self.__pad_right(j.Empty(random_id(), Space.EMPTY, Markers.EMPTY), Space.EMPTY)],
             self.__whitespace()
         )
-        assert self._cursor == len(self._source)
+        # assert self._cursor == len(self._source)
         return cu
 
 
@@ -1591,6 +1611,8 @@ class ParserVisitor(ast.NodeVisitor):
     def __convert_type_hint(self, node) -> Optional[TypeTree]:
         if isinstance(node, ast.Constant):
             literal = cast(j.Literal, self.__convert(node))
+            if node.value is None or node.value is Ellipsis:
+                return literal
             return j.Identifier(
                 random_id(),
                 literal.prefix,
@@ -1687,13 +1709,11 @@ class ParserVisitor(ast.NodeVisitor):
         return ident_or_field(name.split('.'))
 
 
-    def __next_lexer_token(self):
-        tokens = tokenize(BytesIO(self._source[self._cursor:].encode('utf-8')).readline)
-        next(tokens)  # skip ENCODING token
+    def __next_lexer_token(self, tokens: Iterator[TokenInfo]) -> TokenInfo:
         tok = next(tokens)
         value_source = tok.string
         self._cursor += len(value_source)
-        return value_source
+        return tok
 
 
     def __convert_all(self, trees: Sequence) -> List[J2]:
