@@ -1,5 +1,5 @@
 import ast
-import traceback
+import logging
 from pathlib import Path
 from typing import Iterable, Optional
 
@@ -8,6 +8,8 @@ from rewrite.parser import require_print_equals_input, ParserBuilder
 from ._parser_visitor import ParserVisitor
 from .tree import CompilationUnit
 
+logging.basicConfig(level=logging.ERROR)
+
 
 class PythonParser(Parser):
     def parse_inputs(self, sources: Iterable[ParserInput], relative_to: Optional[Path],
@@ -20,7 +22,7 @@ class PythonParser(Parser):
                 cu = ParserVisitor(source_str).visit(tree)
                 cu = require_print_equals_input(self, cu, source, relative_to, ctx)
             except Exception as e:
-                traceback.print_exc()
+                logging.error(f"An error was encountered while parsing {source.path}: {str(e)}", exc_info=True)
                 cu = ParseError.build(self, source, relative_to, ctx, e)
             yield cu
 
